package com.rjconsultores.ventaboletos.web.gui.controladores.relatorios;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Calendar;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.sql.DataSource;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Controller;
import org.zkoss.util.resource.Labels;
import org.zkoss.zhtml.Messagebox;
import org.zkoss.zk.ui.Component;
import org.zkoss.zk.ui.event.Event;
import org.zkoss.zul.Comboitem;
import org.zkoss.zul.Datebox;
import org.zkoss.zul.Paging;
import org.zkoss.zul.Radio;

import com.rjconsultores.ventaboletos.entidad.Empresa;
import com.rjconsultores.ventaboletos.entidad.PuntoVenta;
import com.rjconsultores.ventaboletos.relatorios.impl.RelatorioRemarcacaoDevolucao;
import com.rjconsultores.ventaboletos.relatorios.utilitarios.Relatorio;
import com.rjconsultores.ventaboletos.service.ConstanteService;
import com.rjconsultores.ventaboletos.service.EmpresaService;
import com.rjconsultores.ventaboletos.utilerias.UsuarioLogado;
import com.rjconsultores.ventaboletos.web.utilerias.MyComboboxEstandar;
import com.rjconsultores.ventaboletos.web.utilerias.MyGenericForwardComposer;
import com.rjconsultores.ventaboletos.web.utilerias.MyListbox;
import com.rjconsultores.ventaboletos.web.utilerias.MyTextbox;
import com.rjconsultores.ventaboletos.web.utilerias.paginacion.HibernateSearchObject;
import com.rjconsultores.ventaboletos.web.utilerias.paginacion.PagedListWrapper;
import com.rjconsultores.ventaboletos.web.utilerias.render.RenderRelatorioVendasBilheteiro;
import com.rjconsultores.ventaboletos.web.utilerias.render.RenderRelatorioVendasBilheteiroSelecionados;

@Controller("relatorioRemarcacaoDevolucaoController")
@Scope("prototype")
public class RelatorioRemarcacaoDevolucaoController extends MyGenericForwardComposer {

	private static final long serialVersionUID = 1L;

	private Datebox datInicial;
	private Datebox datFinal;
	private MyComboboxEstandar cmbEmpresa;
	private List<Empresa> lsEmpresa;
	private Paging pagingPuntoVenta;
	private MyTextbox txtNombrePuntoVenta;
	private MyListbox puntoVentaList;
	private MyListbox puntoVentaSelList;

	private Radio rdbTipoRemarcao;

	@Autowired
	private EmpresaService empresaService;
	@Autowired
	private ConstanteService constanteService;
	@Autowired
	private DataSource dataSourceRead;
	@Autowired
	private transient PagedListWrapper<PuntoVenta> plwPuntoVenta;

	@Override
	public void doAfterCompose(Component comp) throws Exception {
		lsEmpresa = empresaService.obtenerTodos();
		super.doAfterCompose(comp);
		puntoVentaList.setItemRenderer(new RenderRelatorioVendasBilheteiro());
		puntoVentaSelList.setItemRenderer(new RenderRelatorioVendasBilheteiroSelecionados());
		rdbTipoRemarcao.setSelected(true);
	}

	/**
	 * @throws Exception
	 * 
	 */
	@SuppressWarnings({ "rawtypes", "unchecked" })
	private void executarRelatorio() throws Exception {
		{
			if (datInicial != null && datFinal != null && datFinal.getValue().compareTo(datInicial.getValue()) < 0) {
				try {
					Messagebox.show(Labels.getLabel("MSG.ningunRegistro"),
							Labels.getLabel("relatorioCancelamentoVendaCartaoController.window.title"),
							Messagebox.OK, Messagebox.INFORMATION);
				} catch (InterruptedException ex) {
					ex.printStackTrace();
				}
			} else
			{
				Relatorio relatorio;

				Map<String, Object> parametros = new HashMap<String, Object>();

				String titulo = rdbTipoRemarcao.isSelected() ?
						Labels.getLabel("relatorioRemarcacaoDevolucao.relatorioRemarcacao.title")
						: Labels.getLabel("relatorioRemarcacaoDevolucao.relatorioDevolucao.title");

				StringBuilder filtro = new StringBuilder();

				filtro.append("Início período: ");
				Calendar cal = Calendar.getInstance();
				cal.setTime(datInicial.getValue());
				filtro.append(cal.get(Calendar.DATE) + "/");
				filtro.append((cal.get(Calendar.MONTH) + 1) + "/");
				filtro.append(cal.get(Calendar.YEAR) + "; ");

				filtro.append("Fim período: ");
				cal.setTime(datFinal.getValue());
				filtro.append(cal.get(Calendar.DATE) + "/");
				filtro.append((cal.get(Calendar.MONTH) + 1) + "/");
				filtro.append(cal.get(Calendar.YEAR) + "; ");

				parametros.put("DATA_INICIAL", (java.util.Date) datInicial.getValue());
				parametros.put("DATA_FINAL", (java.util.Date) datFinal.getValue());

				parametros.put("TITULO", titulo);

				parametros.put("USUARIO", UsuarioLogado.getUsuarioLogado().getUsuarioId().toString());
				parametros.put("USUARIO_NOME", UsuarioLogado.getUsuarioLogado().getNombusuario());

				filtro.append("Empresa: ");
				Comboitem itemEmpresa = cmbEmpresa.getSelectedItem();
				if (itemEmpresa != null) {
					Empresa empresa = (Empresa) itemEmpresa.getValue();
					parametros.put("EMPRESA_ID", empresa.getEmpresaId());
					filtro.append(empresa.getNombempresa() + ";");
				} else {
					filtro.append(" Todas;");
				}

				filtro.append("Agência: ");
				String puntoVentas = "";
				List<PuntoVenta> lsPuntoVentaSelecionados = new ArrayList(Arrays.asList(puntoVentaSelList.getData()));
				if (lsPuntoVentaSelecionados.isEmpty()) {
					puntoVentas = "Todas";
				} else {
					StringBuilder puntoVentaIds = new StringBuilder();
					for (int i = 0; i < lsPuntoVentaSelecionados.size(); i++) {
						PuntoVenta puntoVenta = lsPuntoVentaSelecionados.get(i);
						puntoVentas = puntoVentas + puntoVenta.getNombpuntoventa() + ",";
						if (puntoVentaIds.length() > 0) {
							puntoVentaIds.append(",");
						}
						puntoVentaIds.append(puntoVenta.getPuntoventaId());
					}
					parametros.put("NUMPUNTOVENTA", puntoVentaIds.toString());
				}
				filtro.append(puntoVentas).append(";");
				parametros.put("FILTROS", filtro.toString());
				parametros.put("IS_REMARCACAO", rdbTipoRemarcao.isSelected());
				relatorio = new RelatorioRemarcacaoDevolucao(parametros, dataSourceRead.getConnection(), constanteService, rdbTipoRemarcao.isSelected());

				Map args = new HashMap();
				args.put("relatorio", relatorio);

				openWindow("/component/reportView.zul", titulo, args, MODAL);
			}
		}
	}

	public void onClick$btnExecutarRelatorio(Event ev) throws Exception {
		executarRelatorio();
	}

	private void executarPesquisa() {
		HibernateSearchObject<PuntoVenta> puntoVentaBusqueda =
				new HibernateSearchObject<PuntoVenta>(PuntoVenta.class, pagingPuntoVenta.getPageSize());

		puntoVentaBusqueda.addFilterILike("nombpuntoventa", "%" + txtNombrePuntoVenta.getValue() + "%");
		puntoVentaBusqueda.addFilterEqual(ACTIVO, Boolean.TRUE);

		puntoVentaBusqueda.addSortAsc("nombpuntoventa");

		puntoVentaBusqueda.addFilterEqual(ACTIVO, Boolean.TRUE);

		plwPuntoVenta.init(puntoVentaBusqueda, puntoVentaList, pagingPuntoVenta);

		if (puntoVentaList.getData().length == 0) {
			try {
				Messagebox.show(Labels.getLabel("MSG.ningunRegistro"),
						Labels.getLabel("relatorioVendasBilheteiroController.window.title"),
						Messagebox.OK, Messagebox.INFORMATION);
			} catch (InterruptedException ex) {
				ex.printStackTrace();
			}
		}
	}

	public void onClick$btnPesquisa(Event ev) {
		executarPesquisa();
	}

	public void onClick$btnLimpar(Event ev) {
		puntoVentaList.setData(new ArrayList<PuntoVenta>());
		txtNombrePuntoVenta.setText("");
	}

	public void onDoubleClick$puntoVentaSelList(Event ev) {
		PuntoVenta puntoVenta = (PuntoVenta) puntoVentaSelList.getSelected();
		puntoVentaSelList.removeItem(puntoVenta);
	}

	public void onDoubleClick$puntoVentaList(Event ev) {
		PuntoVenta puntoVenta = (PuntoVenta) puntoVentaList.getSelected();
		puntoVentaSelList.addItemNovo(puntoVenta);
	}

	public Datebox getDatInicial() {
		return datInicial;
	}

	public void setDatInicial(Datebox datInicial) {
		this.datInicial = datInicial;
	}

	public Datebox getDatFinal() {
		return datFinal;
	}

	public void setDatFinal(Datebox datFinal) {
		this.datFinal = datFinal;
	}

	public List<Empresa> getLsEmpresa() {
		return lsEmpresa;
	}

	public void setLsEmpresa(List<Empresa> lsEmpresa) {
		this.lsEmpresa = lsEmpresa;
	}

	public MyComboboxEstandar getCmbEmpresa() {
		return cmbEmpresa;
	}

	public void setCmbEmpresa(MyComboboxEstandar cmbEmpresa) {
		this.cmbEmpresa = cmbEmpresa;
	}
}
